=begin
**
** import.rb
** 11 February 2009
** Siebert Lubbe siebertlubbe [ at ] googlemail [ dot ] com
**
** Desc:
** Import data into dradis from various sources
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end
require 'rexml/document'

# FIXME: we can structure this better, although it works for the time being
require 'extensions/import/nessus'
require 'extensions/import/nmap'
require 'extensions/import/qualys'

module Extensions
  # The module presents methods to import from external data sources into Dradis
  # Actions:
  # [+nmap+] Import data from an nmap result file
  class Import < Extensions::Namespace
    include Nessus
    include Nmap
    include Qualys

    #----------------------------------------------------- private methods
    private

    #----------------------------------------------------- Dispatcher implementation
    INFO = {
      :namespace => 'import',
      :description => 'import external resources to dradis',
      :commands => {
        'nmap' => {
          :desc => 'Import nmap results file',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :desc => 'The file that contains the nmap output',
            :regexp => /.*/
          },
          {
            :required => true,
            :label => 'nmap output format',
            :regexp => /normal|grepable|xml/,
            :desc => 'Either normal or grepable'
          },
          {
            :required => false,
            :label => 'parent id',
            :regexp => /\d*/,
            :desc => 'The parent node under which the new nodes will be created'
          }
          ]
        },
        'qualys' => {
          :desc => 'Import qualys results file',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :desc => 'The file that contains the qualys output',
            :regexp => /.*/
          },
          {
            :required => true,
            :label => 'qualys output format',
            :regexp => /xml/,
            :desc => 'Only xml format is supported'
          },
          {
            :required => false,
            :label => 'parent id',
            :regexp => /\d*/,
            :desc => 'The parent node under which the new nodes will be created'
          }
          ]
        },
        'nessus' => {
          :desc => 'Import nessus results file',
          :syntax => [
          {
            :required => true,
            :label => 'file name',
            :desc => 'The file that contains the nessus output',
            :regexp => /.*/
          },
          {
            :required => true,
            :label => 'nessus output format',
            :regexp => /xml/,
            :desc => 'Only xml format is supported'
          },
          {
            :required => false,
            :label => 'parent id',
            :regexp => /\d*/,
            :desc => 'The parent node under which the new nodes will be created'
          }
          ]
        },
        'textfile' => {
          :desc => 'Import the content of a text file as a note',
          :syntax => [
            {
              :required => true,
              :label => 'filename',
              :desc => 'The text file that contains the note',
              :regexp => /.*/
            },
            {
              :required => true,
              :label => 'node id',
              :regexp => /^\d*$/,
              :desc => 'The node that the note should be added to'
            },
            {
              :required => true,
              :label => 'category',
              :regexp => /.*/,
              :desc => 'The category to be stored with this note'
            }
          ]
        }
      }
    }


    def textfile(*args)
      file_name = args[1]
      node_id = args[2]
      category_name = args[3]

      # check if the text file exists on the file system
      return "#{file_name} is not a valid file" if !File.exists?(file_name)

      # check if the parent id exist if it was passed
      puts @controller.request_service(:model_find_node, args[2]).inspect
      return "Node with id #{node_id.to_s} does not exist" if node_id == nil ||
        @controller.request_service(:model_find_node, node_id.to_i).label == "error, this node does not exist in the server!!"

      return "There is no category with the name '#{category_name}'" if !(category = @controller.request_service(:model_find_category_by_name, category_name))
      
      # open file and read contents into variable
      file_handle = File.new(file_name)
      file_content = file_handle.read

      # import the file contents to the datastore
      if @controller.request_service(:model_add_note, node_id, "Textfile", category.id, file_content)
        return "The text file was successfully imported"
      else
        return "An error occured while importing the text file"
      end
    end

  end
end
