module Nessus

    public
    # Implements the Nessus results import functionality. See above for parameters
    def nessus(*args)
      return "#{args[1]} is not a valid file" if !File.exists?(args[1])

      # allow 0 as a parent id but turn it into a nil
      args[3] = (args[3] == '0' || args[3] == nil) ? nil : args[3].to_i

      # check if the parent id exist if it was passed
      return "Parent node with id #{args[3].to_s} does not exist" if args[3] != nil &&
        @controller.request_service(:model_find_node, args[3]).label == "error, this node does not exist in the server!!"

      # open file and read contents into variable
      file_handle = File.new(args[1])
      file_content = file_handle.readlines

      case args[2]
      when "xml"
        hosts = parse_nessusxml_output(file_content)
      end

      # get the "nessus output" category instance or create it if it does not exist
      category = @controller.request_service(:model_find_category_by_name, "Nessus output") || @controller.request_service(:model_add_category, "Nessus output")

      vuln_notes_to_add = {}
  	  host_root = @controller.request_service(:model_add_node, args[3], nil, "Nessus")
      @controller.request_service(:model_knowledge_refresh)

      hosts.each do |host, detail|
        host_node = @controller.request_service(:model_add_node, host_root.id, nil, host)

        # refresh the local copy of the knowledge cache to get the id of the new node in the knowledge cache
        @controller.request_service(:model_knowledge_refresh)

        # add the nessus output for the host as notes to the node
        @controller.request_service(:model_add_note, host_node.id, "Nessus", category.id, detail[:notes])
        detail[:vulns].each do |vuln, detail|
          vuln_node = @controller.request_service(:model_add_node, host_node.id, nil, "#{vuln}/ Severity:#{detail[:severity]} ")
          # we need the id of the added
          vuln_notes_to_add[vuln_node.id] = "Severity: #{detail[:severity]},\nPlugin ID: #{detail[:pluginID]},\nPlugin Name: #{detail[:pliuginName]},\nData: #{detail[:data]}"
        end
      end

      # refresh the local copy of the knowledge cache to get the ids of the new nodes in the knowledge cache
      @controller.request_service(:model_knowledge_refresh)
      vuln_notes_to_add.each do |vuln_node_id, note|
        @controller.request_service(:model_add_note, vuln_node_id, "Nessus", category.id, note)
      end

      return "#{args[1]} was successfully imported"
    end # def nessus

    private
    def parse_nessusxml_output(content)
      # initiate some variables to be used
      hosts = {}
      xml = REXML::Document.new(content.to_s)
      # loop through all hosts
      REXML::XPath.match(xml[0],"//ReportHost").each do |host|
        hostnodename = host.elements['HostName'].text

    		netbios = host.elements['netbios_name'].text
        mac = host.elements['mac_addr'].text
        dns = host.elements['dns_name'].text
        os = host.elements['os_name'].text

        hosts[hostnodename] = {:notes => hostnodename.to_s}
        hosts[hostnodename][:vulns] = {}
    		hosts[hostnodename][:notes] = "Netbios_name: #{netbios},\nMac_Address: #{mac},\nDNS_Name: #{dns},\nOS: #{os}"

		    REXML::XPath.match(REXML::Document.new(host.to_s),"//ReportItem").each do |vuln|
		    	hosts[hostnodename][:vulns][vuln.elements['port'].text] = {
				    :severity => vuln.elements['severity'].text, 
    				:pluginID => vuln.elements['pluginID'].text, 
		    		:pluginName => vuln.elements['pluginName'].text,				
				    :data => vuln.elements['data'].text
    			}
		    end
      end

      return hosts
    end

end # Nessus
