module Nmap

    public 
    # Implements the nmap results import functionality. See above for parameters
    def nmap(*args)
      return "#{args[1]} is not a valid file" if !File.exists?(args[1])

      # allow 0 as a parent id but turn it into a nil
      args[3] = (args[3] == '0' || args[3] == nil) ? nil : args[3].to_i
      # check if the parent id exist if it was passed
      return "Parent node with id #{args[3].to_s} does not exist" if args[3] != nil &&
        @controller.request_service(:model_find_node, args[3]).label == "error, this node does not exist in the server!!"

      # open file and read contents into variable
      file_handle = File.new(args[1])
      file_content = file_handle.readlines

      case args[2]
      when "grepable"
        hosts = parse_grepable_output(file_content)
      when "normal"
        hosts = parse_normal_output(file_content)
      when "xml"
        hosts = parse_xml_output(file_content)
      end


      # get the "nmap output" category instance or create it if it does not exist
      category = @controller.request_service(:model_find_category_by_name, "Nmap output") || @controller.request_service(:model_add_category, "Nmap output")

      port_notes_to_add = {}
      hosts.each do |host, detail|
        host_node = @controller.request_service(:model_add_node, args[3], nil, host)
        # refresh the local copy of the knowledge cache to get the id of the new node in the knowledge cache
        @controller.request_service(:model_knowledge_refresh)
        # add the nmap output for the host as notes to the node
        @controller.request_service(:model_add_note, host_node.id, "Nmap", category.id, detail[:notes])
        detail[:ports].each do |port, detail|
          port_node = @controller.request_service(:model_add_node, host_node.id, nil, "#{port}/#{detail[:protocol]}")
          # we need the id of the added
          port_notes_to_add[port_node.id] = "State: #{detail[:state]}, Service: #{detail[:service]}"
        end
      end

      # refresh the local copy of the knowledge cache to get the ids of the new nodes in the knowledge cache
      @controller.request_service(:model_knowledge_refresh)
      port_notes_to_add.each do |port_node_id, note|
        @controller.request_service(:model_add_note, port_node_id, "Nmap", category.id, note)
      end

      return "#{args[1]} was successfully imported"
      #return hosts.inspect

    end # def nmap
    
    private
    # parse nmap output that is in normal format
    def parse_normal_output(file_content)
      # initiate some variables to be used
      hosts = {}
      host = ""
      leading_line = ""
      # loop through the contents of the file and populate the hosts hash variable
      file_content.each do |line|
        case line.strip
        when /^Starting Nmap/
          leading_line = line
        when /^Interesting ports on (.*):/
          host = $1
          hosts[host] = {:notes => leading_line+line}
          hosts[host][:ports] = {}
        when /^(\d*)\/(tcp|udp) *(open|filtered|closed|unfiltered) *(.*)/
          hosts[host][:ports][$1] = {:protocol => $2, :state => $3, :service => $4}
          hosts[host][:notes] << line
        when /^PORT *STATE *SERVICE/
          hosts[host][:notes] << line
        when /^Not shown/
          hosts[host][:notes] << line
        else
          next
        end
      end
      return hosts
    end

    # parse nmap output that is in grepable format
    def parse_grepable_output(file_content)
      # initiate some variables to be used
      hosts = {}
      host = ""
      leading_line = ""
      # loop through the contents of the file and populate the hosts hash variable
      file_content.each do |line|
        case line.strip
        when /^# Nmap.*scan/
          leading_line << line
        when /^# Ports scanned/
          leading_line << line
        when /^Host:(.*)Ports:/
          host = $1
          hosts[host] = {:notes => leading_line+line}
          hosts[host][:ports] = {}
          hosts[host][:notes] << line
          ports = line.split(", ")
          ports.each do |port|
            port =~ /(\d*)\/(open|filtered|closed|unfiltered)\/(tcp|upd)\/.*\/(.*)\/.*\/(.*)\//
            hosts[host][:ports][$1] = {:protocol => $3, :state => $2, :service => $4+$5}
          end
        else
          next
        end
      end
      return hosts
    end

    def parse_xml_output(content)
      # initiate some variables to be used
      hosts = {}
      xml = REXML::Document.new(content.to_s)
      
      # loop through all hosts
      REXML::XPath.match(xml[6],"//host").each do |host|
        hostnodename = host.elements['address'].attributes['addr']
        if (!host.elements['hostnames'].elements.empty?)
         hostnodename << '('
         hostnodename << host.elements['hostnames'].elements.to_a.collect do |hostname|
           hostname.attributes['name']
         end.join(",")
         hostnodename << ')'
        end

        hosts[hostnodename] = {:notes => host.to_s}
        hosts[hostnodename][:ports] = {}

        REXML::XPath.match(host.elements['ports'], "port").each do |port|
          hosts[hostnodename][:ports][port.attributes['portid']] = {
            :protocol => port.attributes['protocol'],
            :state => port.elements['state'].attributes['state'],
            :service => port.elements['service'].attributes['name']
          }
        end
      end
      
      return hosts
    end
end # Nmap

