module Qualys
    public
    # Implements the qualys results import functionality. See above for parameters
    def qualys(*args)
      return "#{args[1]} is not a valid file" if !File.exists?(args[1])

	  print Time.now
      # allow 0 as a parent id but turn it into a nil
      args[3] = (args[3] == '0' || args[3] == nil) ? nil : args[3].to_i
      # check if the parent id exist if it was passed
      return "Parent node with id #{args[3].to_s} does not exist" if args[3] != nil &&
        @controller.request_service(:model_find_node, args[3]).label == "error, this node does not exist in the server!!"

      # open file and read contents into variable
      file_handle = File.new(args[1])
      file_content = file_handle.readlines

      case args[2]
      when "xml"
        hosts = parse_qualysxml_output(file_content)
      end

      # get the "qualys output" category instance or create it if it does not exist
      category = @controller.request_service(:model_find_category_by_name, "Qualys output") || @controller.request_service(:model_add_category, "Qualys output")


      info_notes_to_add = {}
      service_notes_to_add = {}
      vuln_notes_to_add = {}

  	  host_root = @controller.request_service(:model_add_node, args[3], nil, "Qualys")
      @controller.request_service(:model_knowledge_refresh)
      hosts.each do |host, detail|
        host_node = @controller.request_service(:model_add_node, host_root.id, nil, host)
  
        # refresh the local copy of the knowledge cache to get the id of the new node in the knowledge cache
        @controller.request_service(:model_knowledge_refresh)
  
        # add the qualys output for the host as notes to the node
        @controller.request_service(:model_add_note, host_node.id, "Qualys", category.id, detail[:notes])
	    
      	infos_node = @controller.request_service(:model_add_node, host_node.id, nil, "Infos")
    		services_node = @controller.request_service(:model_add_node, host_node.id, nil, "Services")
		    vulns_node = @controller.request_service(:model_add_node, host_node.id, nil, "Vulns")

        @controller.request_service(:model_knowledge_refresh)
  
        # Once the Infos sub-folder has been created we need to add the 
        # information associated with it
        cat_vect ={} 

    		detail[:infos].each do |cat, details|
          cat_node = @controller.request_service(:model_add_node, infos_node.id, nil, cat)
    		  cat_vect[cat_node.id] =details 
		    end

        @controller.request_service(:model_knowledge_refresh)
    		cat_vect.each do |cat, details|
          # we need the id of the added
          details.each do |info,data|
          	info_node = @controller.request_service(:model_add_node, cat, nil, info)
      			info_notes_to_add[info_node.id] = data
		      end
        end

        # Once the Services sub-folder has been created we need to add the 
        # information associated with it

        cat_vect = {}
    		detail[:services].each do |cat, details|
          cat_node = @controller.request_service(:model_add_node, services_node.id, nil, cat)
		      cat_vect[cat_node.id] =details 
    		end

        @controller.request_service(:model_knowledge_refresh)
		    cat_vect.each do |cat,details|
          # we need the id of the added
          details.each do |service,data|
          	service_node = @controller.request_service(:model_add_node, cat, nil, service)
			      service_notes_to_add[service_node.id] = data
    		  end
        end
      
        # Once the Vulns sub-folder has been created we need to add the 
        # information associated with it

    		cat_vect = {}
		    detail[:vulns].each do |cat, details|
          cat_node = @controller.request_service(:model_add_node, vulns_node.id, nil, cat)
    		  cat_vect[cat_node.id] = details
		    end

        @controller.request_service(:model_knowledge_refresh)
    		cat_vect.each do |cat,details|
          # we need the id of the added
          details.each do |vuln,data|
          	vuln_node = @controller.request_service(:model_add_node, cat, nil, vuln)
		      	vuln_notes_to_add[vuln_node.id] = data
    		  end
        end
      end

      # refresh the local copy of the knowledge cache to get the ids of the new nodes in the knowledge cache
      @controller.request_service(:model_knowledge_refresh)
      info_notes_to_add.each do |info_node_id, note|
        @controller.request_service(:model_add_note, info_node_id, "Qualys", category.id, note)
      end
      service_notes_to_add.each do |service_node_id, note|
        @controller.request_service(:model_add_note, service_node_id, "Qualys", category.id, note)
      end
      vuln_notes_to_add.each do |vuln_node_id, note|
        @controller.request_service(:model_add_note, vuln_node_id, "Qualys", category.id, note)
      end

      print Time.now

      return "#{args[1]} was successfully imvulned"
    end # def qualys


    private
    def parse_qualysxml_output(content)
      # initiate some variables to be used
      hosts = {}
	  
      xml = REXML::Document.new(content.to_s)
      # loop through all hosts
      REXML::XPath.match(xml[3],"//IP").each do |host|
        #hostnodename = host.elements['HostName'].attributes['name']
        hostnodename = host.attributes['name']

    		if host.elements['OS']
		    	os = host.elements['OS'].text
    		else
		    	os = "Not found"
    		end

		    ip = host.attributes['value']
    		domain = host.attributes['name']
        hosts[hostnodename] = {:notes => hostnodename.to_s}
		    hosts[hostnodename][:notes] = "IP: #{ip},\nDomain: #{domain},\nOS: #{os}"

        # so far we have host and general information. We need to add Infos, 
        # Vulns and Services
        hosts[hostnodename][:infos] = {}
        hosts[hostnodename][:services] = {}
        hosts[hostnodename][:vulns] = {}

        #---------------------------------------------------------------- Infos
		    REXML::XPath.match(REXML::Document.new(host.to_s),"//INFOS").each do |infos|
			    REXML::XPath.match(REXML::Document.new(infos.to_s),"//CAT").each do |cat|
    				hosts[hostnodename][:infos][cat.attributes['value']] = {}
		    		REXML::XPath.match(REXML::Document.new(cat.to_s),"//INFO").each do |info|

    					msg = "Data:\n"
		    			if cat.attributes['port']
				    		msg = msg + "Port: " + cat.attributes['port'] + "\n"
    					end
		    			if cat.attributes['protocol']
				    		msg = msg + "Protocol: " + cat.attributes['protocol'] + "\n"
    					end
		    			if cat.attributes['misc']
				    		msg = msg + "Misc: " + cat.attributes['misc'] + "\n"
    					end
		    			if info.elements['PCI_FLAG']
				    		msg = msg + "\nPci_Flag: " + info.elements['PCI_FLAG'].text + "\n"
    					end
		    			if info.attributes['severity']
				    		msg = msg + "Severity: " + info.attributes['severity'] + "\n"
    					end
		    			if info.elements['DIAGNOSIS']
				    		msg = msg + "Diagnosis: " + info.elements['DIAGNOSIS'].text + "\n"
    					end
		    			if info.elements['CONSEQUENCE']
				    		msg = msg + "Consequence: " + info.elements['CONSEQUENCE'].text + "\n"
    					end
		    			if info.elements['RESULT']
				    		msg = msg + "Result: " + info.elements['RESULT'].text + "\n"
    					end
		    			if info.elements['SOLUTION']
				    		msg = msg + "Solution: " + info.elements['SOLUTION'].text + "\n"
    					end
		    			hosts[hostnodename][:infos][cat.attributes['value']][info.elements['TITLE'].text] = msg
    				end
		    	end	
    		end

        #------------------------------------------------------------- Services
    		REXML::XPath.match(REXML::Document.new(host.to_s),"//SERVICES").each do |services|
		    	REXML::XPath.match(REXML::Document.new(services.to_s),"//CAT").each do |cat|
				    hosts[hostnodename][:services][cat.attributes['value']] = {}
    				REXML::XPath.match(REXML::Document.new(cat.to_s),"//SERVICE").each do |service|
		    			msg = "Data:\n"
				    	if cat.attributes['port']
    						msg = msg + "Port: " + cat.attributes['port'] + "\n"
		    			end
				    	if cat.attributes['protocol']
						    msg = msg + "Protocol: " + cat.attributes['protocol'] + "\n"
    					end
    					if cat.attributes['misc']
		    				msg = msg + "Misc: " + cat.attributes['misc'] + "\n"
				    	end
    					if service.elements['PCI_FLAG']
		    				msg = msg + "\nPci_Flag: " + service.elements['PCI_FLAG'].text + "\n"
				    	end
    					if service.attributes['severity']
		    				msg = msg + "Severity: " + service.attributes['severity'] + "\n"
				    	end
    					if service.elements['DIAGNOSIS']
		    				msg = msg + "Diagnosis: " + service.elements['DIAGNOSIS'].text + "\n"
				    	end
    					if service.elements['CONSEQUENCE']
		    				msg = msg + "Consequence: " + service.elements['CONSEQUENCE'].text + "\n"
    					end
		    			if service.elements['RESULT']
				    		msg = msg + "Result: " + service.elements['RESULT'].text + "\n"
    					end
		    			if service.elements['SOLUTION']
				    		msg = msg + "Solution: " + service.elements['SOLUTION'].text + "\n"
    					end
		    			hosts[hostnodename][:services][cat.attributes['value']][service.elements['TITLE'].text] = msg
    				end
		    	end	
    		end

        #---------------------------------------------------------------- Vulns
    		REXML::XPath.match(REXML::Document.new(host.to_s),"//VULNS").each do |vulns|
		    	REXML::XPath.match(REXML::Document.new(vulns.to_s),"//CAT").each do |cat|
				    hosts[hostnodename][:vulns][cat.attributes['value']] = {}
    				REXML::XPath.match(REXML::Document.new(cat.to_s),"//VULN").each do |vuln|
		    			msg = "Data:\n"
				    	if cat.attributes['port']
						    msg = msg + "Port: " + cat.attributes['port'] + "\n"
    					end
    					if cat.attributes['protocol']
		    				msg = msg + "Protocol: " + cat.attributes['protocol'] + "\n"
				    	end
    					if cat.attributes['misc']
		    				msg = msg + "Misc: " + cat.attributes['misc'] + "\n"
				    	end
    					if vuln.elements['PCI_FLAG']
		    				msg = msg + "\nPci_Flag: " + vuln.elements['PCI_FLAG'].text + "\n"
				    	end
    					if vuln.attributes['severity']
		    				msg = msg + "Severity: " + vuln.attributes['severity'] + "\n"
				    	end
    					if vuln.attributes['cveid']
		    				msg = msg + "CVE list: " + vuln.attributes['cveid'] + "\n"
				    	end
    					if vuln.elements['DIAGNOSIS']
		    				msg = msg + "Diagnosis: " + vuln.elements['DIAGNOSIS'].text + "\n"
				    	end
    					if vuln.elements['CONSEQUENCE']
		    				msg = msg + "Consequence: " + vuln.elements['CONSEQUENCE'].text + "\n"
				    	end
    					if vuln.elements['RESULT']
		    				msg = msg + "Result: " + vuln.elements['RESULT'].text + "\n"
				    	end
    					if vuln.elements['SOLUTION']
		    				msg = msg + "Solution: " + vuln.elements['SOLUTION'].text + "\n"
				    	end
    					hosts[hostnodename][:vulns][cat.attributes['value']][vuln.elements['TITLE'].text] = msg
    				end
		    	end	
    		end

		    print hostnodename
    		puts
      end

      return hosts
    end


end # Qualys
