=begin
**
** wx.rb
** 01/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  This script module contains all the User Interface related code using the 
** wxWidgets implementation
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

# first check that the gem is installed, if it is not, provide some helpful
# clue. See Rails' activesupport/lib/active_support/core_ext/kernel/requires.rb
silence_warnings do
  begin
    require 'wx'
  rescue LoadError => cannot_require
    # 1. Requiring the module is unsuccessful, maybe it's a gem and nobody 
    #required rubygems yet. Try.
    begin
      require 'rubygems'
    rescue LoadError => rubygems_not_installed
      $logger.fatal{ 'wxRuby not found on the system and RubyGems in not available.' }
      $logger.fatal{ 'Please install RubyGems and the wxruby gem (gem install wxruby). Version 1.9.9 or higher of the gem is required.' }
      raise rubygems_not_installed
    end

    # 2. Rubygems is installed and loaded. Try to load the library again
    begin
      unless ( gem('wxruby', '>=1.9.9') )
        $logger.fatal{ 'Unexpected error while loading the wxruby gem.' }
        if (RUBY_PLATFORM.match('linux'))
          $logger.fatal{ 'This is a know issue in Debian systems with wxruby 2.0.1. Try:' }
          $logger.fatal{ '# gem uninstall wxruby -a'  }
          $logger.fatal{ '# gem install wxruby -v=2.0.0' }
        end
        raise 
      end
    rescue LoadError => gem_not_installed
      $logger.fatal{ 'wxRuby not found on the system and gem wxruby not installed' }
      $logger.fatal{ 'Please install the wxruby gem (gem install wxruby). Version 1.9.9 or higher of the gem is required.' }
      raise cannot_require
    end
  end

  # if the library is found (compiled version, or provided by the OS packet
  # manager, we need to check for the version)
  requirement = Gem::Requirement.create('>=1.9.9')
  version = Gem::Version.create(Wx::WXRUBY_VERSION)

  unless requirement.satisfied_by?(version)
    msg = "The version of the wxruby library (#{Wx::WXRUBY_VERSION}) is not " +
      'supported. Please install version 1.9.9 or higher using: gem install wxruby'
    $logger.error(msg)
    raise LoadError.new(msg)
  end

end

require 'ui/dialogs'
require 'ui/widgets'
require 'ui/wx/window'

# Different user interfaces will be grouped under this module.
module Ui
  # Graphical user interface classes and modules. Based on the wxWidgets
  # framework (http://www.wxwidgets.org/). In order for this to work you will 
  # need the wxruby gem (http://wxruby.rubyforge.org/) installed.
  module WxWidgets
    # Main Wx Application, handles the creation of main window, window title, etc.
    class DradisGui < Wx::App
      include Core::Interfaces::View
      
      def on_init
        @frame = Ui::WxWidgets::DradisWindow.new( nil, 
                           Wx::ID_ANY, 
                           Core::VERSION::string,
                           Wx::DEFAULT_POSITION,
                           Wx::Size.new(800, 700) 
                        )
        set_top_window(@frame)
        @frame.controller = @controller
        @frame.show
        return true                       
      end
      def setup(new_controller)
        @controller = new_controller
        # register the view as an observer of the exceptions thrown by the 
        # controller
        @controller.add_observer(:exception, self)
      end
      def run()      
        main_loop
      end
      def teardown()
        @frame.teardown
      end
      
      def notify(event, *args)
        if @frame.nil?
          # an error accured during initialisation
          $logger.error{ event }
          $logger.error{ args }
        else
          @frame.notify(event, *args)
        end
      end
    end
  end
end
